@@ -16,12 +16,16 @@
  */
 package org.apache.activemq.artemis.api.core;
 
+import javax.management.openmbean.CompositeData;
+import javax.management.openmbean.OpenDataException;
 import java.io.InputStream;
 import java.util.HashMap;
 import java.util.Map;
 import java.util.Set;
+import java.util.function.Predicate;
 
 import io.netty.buffer.ByteBuf;
+import org.apache.activemq.artemis.core.persistence.CoreMessageObjectPools;
 import org.apache.activemq.artemis.core.persistence.Persister;
 
 /**
@@ -70,12 +74,17 @@ import org.apache.activemq.artemis.core.persistence.Persister;
  */
 public interface Message {
 
-   // This is an estimate of how much memory a Message takes up, exclusing body and properties
+   // This is an estimate of how much memory a Message takes up, excluding body and properties
    // Note, it is only an estimate, it's not possible to be entirely sure with Java
    // This figure is calculated using the test utilities in org.apache.activemq.tests.unit.util.sizeof
    // The value is somewhat higher on 64 bit architectures, probably due to different alignment
    int memoryOffset = 352;
 
+   // We use properties to establish routing context on clustering.
+   // However if the client resends the message after receiving, it needs to be removed, so we mark these internal
+   Predicate<SimpleString> INTERNAL_PROPERTY_NAMES_PREDICATE =
+      name -> (name.startsWith(Message.HDR_ROUTE_TO_IDS) && !name.equals(Message.HDR_ROUTE_TO_IDS)) ||
+      (name.startsWith(Message.HDR_ROUTE_TO_ACK_IDS) && !name.equals(Message.HDR_ROUTE_TO_ACK_IDS));
 
    SimpleString HDR_ROUTE_TO_IDS = new SimpleString("_AMQ_ROUTE_TO");
 
@@ -93,7 +102,7 @@ public interface Message {
    SimpleString HDR_ACTUAL_EXPIRY_TIME = new SimpleString("_AMQ_ACTUAL_EXPIRY");
 
    /**
-    * The original address of a message when a message is transferred through DLQ or expiry
+    * The original address of a message when a message is diverted or transferred through DLQ or expiry
     */
    SimpleString HDR_ORIGINAL_ADDRESS = new SimpleString("_AMQ_ORIG_ADDRESS");
 
@@ -103,7 +112,7 @@ public interface Message {
    SimpleString HDR_ORIGINAL_QUEUE = new SimpleString("_AMQ_ORIG_QUEUE");
 
    /**
-    * The original message ID before th emessage was transferred.
+    * The original message ID before the message was transferred.
     */
    SimpleString HDR_ORIG_MESSAGE_ID = new SimpleString("_AMQ_ORIG_MESSAGE_ID");
 
@@ -112,6 +121,8 @@ public interface Message {
     */
    SimpleString HDR_GROUP_ID = new SimpleString("_AMQ_GROUP_ID");
 
+   SimpleString HDR_GROUP_SEQUENCE = new SimpleString("_AMQ_GROUP_SEQUENCE");
+
    /**
     * to determine if the Large Message was compressed.
     */
@@ -152,6 +163,22 @@ public interface Message {
     */
    SimpleString HDR_ROUTING_TYPE = new SimpleString("_AMQ_ROUTING_TYPE");
 
+   /**
+    * The original routing type of a message before getting transferred through DLQ or expiry
+    */
+   SimpleString HDR_ORIG_ROUTING_TYPE = new SimpleString("_AMQ_ORIG_ROUTING_TYPE");
+
+   /**
+    * The time at which the message arrived at the broker.
+    */
+   SimpleString HDR_INGRESS_TIMESTAMP = new SimpleString("_AMQ_INGRESS_TIMESTAMP");
+
+   /**
+    * The prefix used (if any) when sending this message.  For protocols (e.g. STOMP) that need to track this and restore
+    * the prefix when the message is consumed.
+    */
+   SimpleString HDR_PREFIX = new SimpleString("_AMQ_PREFIX");
+
    byte DEFAULT_TYPE = 0;
 
    byte OBJECT_TYPE = 2;
@@ -164,16 +191,40 @@ public interface Message {
 
    byte STREAM_TYPE = 6;
 
-   default void cleanupInternalProperties() {
+   /** The message will contain another message persisted through {@literal org.apache.activemq.artemis.spi.core.protocol.EmbedMessageUtil}*/
+   byte EMBEDDED_TYPE = 7;
+
+   /** This is to embedd Large Messages from other protocol */
+   byte LARGE_EMBEDDED_TYPE = 8;
+
+   default void clearInternalProperties() {
       // only on core
    }
 
-   RoutingType getRouteType();
+   /**
+    * Search for the existence of the property: an implementor can save
+    * the message to be decoded, if possible.
+    */
+   default boolean hasScheduledDeliveryTime() {
+      return getScheduledDeliveryTime() != null;
+   }
+
+   default RoutingType getRoutingType() {
+      return null;
+   }
+
+   default Message setRoutingType(RoutingType routingType) {
+      return this;
+   }
 
    default SimpleString getLastValueProperty() {
       return null;
    }
 
+   default Message setLastValueProperty(SimpleString lastValueName) {
+      return this;
+   }
+
    /**
     * @deprecated do not use this, use through ICoreMessage or ClientMessage
     */
@@ -184,6 +235,8 @@ public interface Message {
 
    /**
     * @deprecated do not use this, use through ICoreMessage or ClientMessage
+    * Warning: if you need to read the content of a message use getDataBuffer(). This method is intended for when you
+    *          want to make changes.
     */
    @Deprecated
    default ActiveMQBuffer getBodyBuffer() {
@@ -206,7 +259,9 @@ public interface Message {
       return this;
    }
 
-
+   /**
+    * WARNING: Calling this method on a AMQPMessage will allow the non mutable part of the message to be modified.
+    */
    void messageChanged();
 
    /** Used to calculate what is the delivery time.
@@ -217,34 +272,57 @@ public interface Message {
       return this;
    }
 
-   /** Context can be used by the application server to inject extra control, like a protocol specific on the server.
-    * There is only one per Object, use it wisely!
-    *
-    * Note: the intent of this was to replace PageStore reference on Message, but it will be later increased by adidn a ServerPojo
-    * */
-   RefCountMessageListener getContext();
-
    default SimpleString getGroupID() {
       return null;
    }
 
+   default Message setGroupID(SimpleString groupID) {
+      return this;
+   }
+
+   default Message setGroupID(String groupID) {
+      return this;
+   }
+
+   default int getGroupSequence() {
+      return 0;
+   }
+
+   default Message setGroupSequence(int sequence) {
+      return this;
+   }
+
+   default Object getCorrelationID() {
+      return null;
+   }
+
+   default Message setCorrelationID(Object correlationID) {
+
+      return this;
+   }
+
    SimpleString getReplyTo();
 
    Message setReplyTo(SimpleString address);
 
-   Message setContext(RefCountMessageListener context);
-
-   /** The buffer will belong to this message, until release is called. */
-   Message setBuffer(ByteBuf buffer);
-
-   ByteBuf getBuffer();
-
    /** It will generate a new instance of the message encode, being a deep copy, new properties, new everything */
    Message copy();
 
    /** It will generate a new instance of the message encode, being a deep copy, new properties, new everything */
    Message copy(long newID);
 
+   /** It will generate a new instance of the message encode, being a deep copy, new properties, new everything */
+   default Message copy(long newID, boolean isExpiryOrDLQ) {
+      return copy(newID);
+   }
+
+   default boolean acceptsConsumer(long uniqueConsumerID) {
+      return true;
+   }
+
+   default void rejectConsumer(long uniqueConsumerID) {
+   }
+
    /**
     * Returns the messageID.
     * <br>
@@ -252,6 +330,8 @@ public interface Message {
     */
    long getMessageID();
 
+   String getProtocolName();
+
    // used for NO-LOCAL: mainly for AMQP
    default Message setConnectionID(String connectionID) {
       return this;
@@ -292,8 +372,11 @@ public interface Message {
 
 
    /**
-    * Returns the userID - this is an optional user specified UUID that can be set to identify the message
-    * and will be passed around with the message
+    *
+    * This represents historically the JMSMessageID.
+    * We had in the past used this for the MessageID that was sent on core messages...
+    *
+    * later on when we added AMQP this name clashed with AMQPMessage.getUserID();
     *
     * @return the user id
     */
@@ -301,6 +384,14 @@ public interface Message {
 
    Message setUserID(Object userID);
 
+   default  String getValidatedUserID() {
+      return null;
+   }
+
+   default Message setValidatedUserID(String validatedUserID) {
+      return this;
+   }
+
    /**
     * Returns whether this message is durable or not.
     */
@@ -317,10 +408,28 @@ public interface Message {
 
    String getAddress();
 
+   /**
+    * Look at {@link #setAddress(SimpleString)} for the doc.
+    * @param address
+    * @return
+    */
    Message setAddress(String address);
 
    SimpleString getAddressSimpleString();
 
+   /**
+    * This will set the address on CoreMessage.
+    *
+    * Note for AMQPMessages:
+    * in AMQPMessages this will not really change the address on the message. Instead it will add a property
+    * on extraProperties which only transverse internally at the broker.
+    * Whatever you change here it won't affect anything towards the received message.
+    *
+    * If you wish to change AMQPMessages address you will have to do it directly at the AMQP Message, however beware
+    * that AMQPMessages are not supposed to be changed at the broker, so only do it if you know what you are doing.
+    * @param address
+    * @return
+    */
    Message setAddress(SimpleString address);
 
    long getTimestamp();
@@ -355,32 +464,19 @@ public interface Message {
 
    void persist(ActiveMQBuffer targetRecord);
 
-   void reloadPersistence(ActiveMQBuffer record);
+   void reloadPersistence(ActiveMQBuffer record, CoreMessageObjectPools pools);
 
-   default void releaseBuffer() {
-      ByteBuf buffer = getBuffer();
-      if (buffer != null) {
-         buffer.release();
-      }
-      setBuffer(null);
+   default void reencode() {
+      // only valid probably on AMQP
    }
-   default void referenceOriginalMessage(final Message original, String originalQueue) {
-      String queueOnMessage = original.getStringProperty(Message.HDR_ORIGINAL_QUEUE.toString());
 
-      if (queueOnMessage != null) {
-         putStringProperty(Message.HDR_ORIGINAL_QUEUE.toString(), queueOnMessage);
-      } else if (originalQueue != null) {
-         putStringProperty(Message.HDR_ORIGINAL_QUEUE.toString(), originalQueue);
-      }
-
-      if (original.containsProperty(Message.HDR_ORIG_MESSAGE_ID.toString())) {
-         putStringProperty(Message.HDR_ORIGINAL_ADDRESS.toString(), original.getStringProperty(Message.HDR_ORIGINAL_ADDRESS.toString()));
-
-         putLongProperty(Message.HDR_ORIG_MESSAGE_ID.toString(), original.getLongProperty(Message.HDR_ORIG_MESSAGE_ID.toString()));
-      } else {
-         putStringProperty(Message.HDR_ORIGINAL_ADDRESS.toString(), original.getAddress());
-
-         putLongProperty(Message.HDR_ORIG_MESSAGE_ID.toString(), original.getMessageID());
+   default void referenceOriginalMessage(final Message original, final SimpleString originalQueue) {
+      setBrokerProperty(Message.HDR_ORIGINAL_QUEUE, originalQueue);
+      setBrokerProperty(Message.HDR_ORIGINAL_ADDRESS, original.getAddressSimpleString());
+      setBrokerProperty(Message.HDR_ORIG_MESSAGE_ID, original.getMessageID());
+      final RoutingType routingType = original.getRoutingType();
+      if (routingType != null) {
+         setBrokerProperty(Message.HDR_ORIG_ROUTING_TYPE, routingType.getType());
       }
 
       // reset expiry
@@ -407,6 +503,19 @@ public interface Message {
       }
    }
 
+
+   default org.apache.activemq.artemis.api.core.Message putExtraBytesProperty(SimpleString key, byte[] value) {
+      return putBytesProperty(key, value);
+   }
+
+   default byte[] getExtraBytesProperty(SimpleString key) throws ActiveMQPropertyConversionException {
+      return getBytesProperty(key);
+   }
+
+   default byte[] removeExtraBytesProperty(SimpleString key) throws ActiveMQPropertyConversionException {
+      return (byte[])removeProperty(key);
+   }
+
    default Object getDuplicateProperty() {
       return null;
    }
@@ -501,11 +610,52 @@ public interface Message {
 
    Long getLongProperty(SimpleString key) throws ActiveMQPropertyConversionException;
 
+   default Object getObjectPropertyForFilter(SimpleString key) {
+      return getObjectProperty(key);
+   }
+
    Object getObjectProperty(SimpleString key);
 
-   Object removeDeliveryAnnotationProperty(SimpleString key);
+   default Object removeAnnotation(SimpleString key) {
+      return removeProperty(key);
+   }
 
-   Object getDeliveryAnnotationProperty(SimpleString key);
+   default String getAnnotationString(SimpleString key) {
+      Object value = getAnnotation(key);
+      if (value != null) {
+         return value.toString();
+      } else {
+         return null;
+      }
+   }
+
+   Object getAnnotation(SimpleString key);
+
+   /** Callers must call {@link #reencode()} in order to be sent to clients */
+   default Message setAnnotation(SimpleString key, Object value) {
+      putObjectProperty(key, value);
+      return this;
+   }
+
+   /** To be called by the broker on ocasions such as DLQ and expiry.
+    * When the broker is adding additional properties. */
+   default Message setBrokerProperty(SimpleString key, Object value) {
+      putObjectProperty(key, value);
+      return this;
+   }
+
+   default Object getBrokerProperty(SimpleString key) {
+      return getObjectProperty(key);
+   }
+
+   default Message setIngressTimestamp() {
+      setBrokerProperty(HDR_INGRESS_TIMESTAMP, System.currentTimeMillis());
+      return this;
+   }
+
+   default Long getIngressTimestamp() {
+      return (Long) getBrokerProperty(HDR_INGRESS_TIMESTAMP);
+   }
 
    Short getShortProperty(SimpleString key) throws ActiveMQPropertyConversionException;
 
@@ -519,11 +669,23 @@ public interface Message {
 
    Message putStringProperty(SimpleString key, SimpleString value);
 
+   Message putStringProperty(SimpleString key, String value);
+
    /**
     * Returns the size of the <em>encoded</em> message.
     */
    int getEncodeSize();
 
+   /**
+    * Return an estimate of the size of the message on the wire.
+    * for LargeMessages this will contain whatever is needed to encode properties and the body size of large messages.
+    * For AMQP this will return the whole body size of the message as the body will contain all the data including properties.
+    * @return
+    */
+   default long getWholeMessageSize() {
+      return getEncodeSize();
+   }
+
    /**
     * Returns all the names of the properties for this message.
     */
@@ -533,26 +695,49 @@ public interface Message {
 
    int getRefCount();
 
-   int incrementRefCount() throws Exception;
+   int getUsage();
 
-   int decrementRefCount() throws Exception;
+   int getDurableCount();
 
-   int incrementDurableRefCount();
+   /** this method indicates usage by components such as large message or page cache.
+    *  This method will cause large messages to be held longer after the ack happened for instance.
+    */
+   int usageUp();
 
-   int decrementDurableRefCount();
+   /**
+    * @see #usageUp()
+    * @return
+    */
+   int usageDown();
+
+   int refUp();
+
+   int refDown();
+
+   int durableUp();
+
+   int durableDown();
 
    /**
     * @return Returns the message in Map form, useful when encoding to JSON
     */
    default Map<String, Object> toMap() {
-      Map map = toPropertyMap();
+      return toMap(-1);
+   }
+
+   /**
+    * @return Returns the message in Map form, useful when encoding to JSON
+    * @param valueSizeLimit that limits [] map values
+    */
+   default Map<String, Object> toMap(int valueSizeLimit) {
+      Map map = toPropertyMap(valueSizeLimit);
       map.put("messageID", getMessageID());
       Object userID = getUserID();
       if (getUserID() != null) {
          map.put("userID", "ID:" + userID.toString());
       }
 
-      map.put("address", getAddress());
+      map.put("address", getAddress() == null ? "" : getAddress());
       map.put("durable", isDurable());
       map.put("expiration", getExpiration());
       map.put("timestamp", getTimestamp());
@@ -565,19 +750,55 @@ public interface Message {
     * @return Returns the message properties in Map form, useful when encoding to JSON
     */
    default Map<String, Object> toPropertyMap() {
+      return toPropertyMap(-1);
+   }
+
+   /**
+    * @return Returns the message properties in Map form, useful when encoding to JSON
+    * @param valueSizeLimit that limits [] map values
+    */
+   default Map<String, Object> toPropertyMap(int valueSizeLimit) {
       Map map = new HashMap<>();
       for (SimpleString name : getPropertyNames()) {
-         map.put(name.toString(), getObjectProperty(name.toString()));
+         Object value = getObjectProperty(name.toString());
+         //some property is SimpleString, which is not available for management console
+         if (value instanceof SimpleString) {
+            value = value.toString();
+         }
+         value = JsonUtil.truncate(value, valueSizeLimit);
+         map.put(name.toString(), value);
       }
       return map;
    }
 
-
    /** This should make you convert your message into Core format. */
    ICoreMessage toCore();
 
+   default CompositeData toCompositeData(int fieldsLimit, int deliveryCount) throws OpenDataException {
+      return null;
+   }
+
+   /** This should make you convert your message into Core format. */
+   ICoreMessage toCore(CoreMessageObjectPools coreMessageObjectPools);
+
    int getMemoryEstimate();
 
+   /**
+    * This is the size of the message when persisted on disk which is used for metrics tracking
+    * Note that even if the message itself is not persisted on disk (ie non-durable) this value is
+    * still used for metrics tracking
+    * If a normal message it will be the encoded message size
+    * If a large message it will be encoded message size + large message body size
+    * @return
+    * @throws ActiveMQException
+    */
+   long getPersistentSize() throws ActiveMQException;
 
+   Object getOwner();
 
+   void setOwner(Object object);
+
+   default String getStringBody() {
+      return null;
+   }
 }
