@@ -27,8 +27,7 @@ import org.apache.activemq.artemis.api.core.ActiveMQException;
 import org.apache.activemq.artemis.api.core.ActiveMQPropertyConversionException;
 import org.apache.activemq.artemis.api.core.ICoreMessage;
 import org.apache.activemq.artemis.api.core.Message;
-import org.apache.activemq.artemis.api.core.RefCountMessage;
-import org.apache.activemq.artemis.api.core.RoutingType;
+import org.apache.activemq.artemis.api.core.QueueConfiguration;
 import org.apache.activemq.artemis.api.core.SimpleString;
 import org.apache.activemq.artemis.api.core.client.ClientConsumer;
 import org.apache.activemq.artemis.api.core.client.ClientMessage;
@@ -38,12 +37,12 @@ import org.apache.activemq.artemis.api.core.client.ClientSessionFactory;
 import org.apache.activemq.artemis.api.core.client.MessageHandler;
 import org.apache.activemq.artemis.api.core.client.ServerLocator;
 import org.apache.activemq.artemis.core.client.impl.ClientSessionInternal;
+import org.apache.activemq.artemis.core.persistence.CoreMessageObjectPools;
 import org.apache.activemq.artemis.core.persistence.Persister;
 import org.apache.activemq.artemis.core.protocol.core.impl.ActiveMQConsumerContext;
 import org.apache.activemq.artemis.core.server.ActiveMQServer;
 import org.apache.activemq.artemis.core.server.Queue;
 import org.apache.activemq.artemis.spi.core.remoting.ConsumerContext;
-import org.apache.activemq.artemis.tests.integration.IntegrationTestLogger;
 import org.apache.activemq.artemis.tests.util.ActiveMQTestBase;
 import org.apache.activemq.artemis.utils.UUID;
 import org.junit.Assert;
@@ -51,8 +50,6 @@ import org.junit.Test;
 
 public class AcknowledgeTest extends ActiveMQTestBase {
 
-   private static final IntegrationTestLogger log = IntegrationTestLogger.LOGGER;
-
    public final SimpleString addressA = new SimpleString("addressA");
 
    public final SimpleString queueA = new SimpleString("queueA");
@@ -69,7 +66,7 @@ public class AcknowledgeTest extends ActiveMQTestBase {
       ClientSessionFactory cf = createSessionFactory(locator);
       ClientSession sendSession = cf.createSession(false, true, true);
       ClientSession session = cf.createSession(false, true, true);
-      sendSession.createQueue(addressA, queueA, false);
+      sendSession.createQueue(new QueueConfiguration(queueA).setAddress(addressA).setDurable(false));
       ClientProducer cp = sendSession.createProducer(addressA);
       ClientConsumer cc = session.createConsumer(queueA);
       int numMessages = 100;
@@ -99,7 +96,7 @@ public class AcknowledgeTest extends ActiveMQTestBase {
       ClientSessionFactory cf = createSessionFactory(locator);
       ClientSession sendSession = cf.createSession(false, true, true);
       ClientSession session = cf.createSession(false, true, true);
-      sendSession.createQueue(addressA, queueA, false);
+      sendSession.createQueue(new QueueConfiguration(queueA).setAddress(addressA).setDurable(false));
       ClientProducer cp = sendSession.createProducer(addressA);
       ClientConsumer cc = session.createConsumer(queueA);
       int numMessages = 3;
@@ -108,7 +105,7 @@ public class AcknowledgeTest extends ActiveMQTestBase {
       }
 
       Thread.sleep(500);
-      log.info("woke up");
+      instanceLog.debug("woke up");
 
       final CountDownLatch latch = new CountDownLatch(numMessages);
       session.start();
@@ -117,7 +114,7 @@ public class AcknowledgeTest extends ActiveMQTestBase {
 
          @Override
          public void onMessage(final ClientMessage message) {
-            log.info("Got message " + c++);
+            instanceLog.debug("Got message " + c++);
             latch.countDown();
          }
       });
@@ -136,7 +133,7 @@ public class AcknowledgeTest extends ActiveMQTestBase {
       ClientSessionFactory cf = createSessionFactory(locator);
       ClientSession sendSession = cf.createSession(false, true, true);
       final ClientSession session = cf.createSession(false, true, true);
-      sendSession.createQueue(addressA, queueA, false);
+      sendSession.createQueue(new QueueConfiguration(queueA).setAddress(addressA).setDurable(false));
       ClientProducer cp = sendSession.createProducer(addressA);
       ClientConsumer cc = session.createConsumer(queueA);
       int numMessages = 100;
@@ -187,7 +184,7 @@ public class AcknowledgeTest extends ActiveMQTestBase {
 
       sessionConsumer.start();
 
-      sessionConsumer.createQueue(addressA, queueA, true);
+      sessionConsumer.createQueue(new QueueConfiguration(queueA).setAddress(addressA));
 
       ClientConsumer consumer = sessionConsumer.createConsumer(queueA);
 
@@ -250,7 +247,7 @@ public class AcknowledgeTest extends ActiveMQTestBase {
       ClientSessionFactory cf = createSessionFactory(locator);
       ClientSession sendSession = cf.createSession(false, true, true);
       final ClientSession session = cf.createSession(false, true, true);
-      sendSession.createQueue(addressA, queueA, false);
+      sendSession.createQueue(new QueueConfiguration(queueA).setAddress(addressA).setDurable(false));
       ClientProducer cp = sendSession.createProducer(addressA);
       ClientConsumer cc = session.createConsumer(queueA);
       int numMessages = 100;
@@ -337,13 +334,14 @@ public class AcknowledgeTest extends ActiveMQTestBase {
       }
    }
 
-   class FakeMessageWithID extends RefCountMessage {
+   class FakeMessageWithID implements Message {
 
       final long id;
 
       @Override
-      public RoutingType getRouteType() {
-         return null;
+      public String getProtocolName() {
+         // should normally not be visible in GUI
+         return getClass().getName();
       }
 
       @Override
@@ -357,12 +355,12 @@ public class AcknowledgeTest extends ActiveMQTestBase {
       }
 
       @Override
-      public Object removeDeliveryAnnotationProperty(SimpleString key) {
+      public Object removeAnnotation(SimpleString key) {
          return null;
       }
 
       @Override
-      public Object getDeliveryAnnotationProperty(SimpleString key) {
+      public Object getAnnotation(SimpleString key) {
          return null;
       }
 
@@ -371,6 +369,11 @@ public class AcknowledgeTest extends ActiveMQTestBase {
          return 0;
       }
 
+      @Override
+      public int getDurableCount() {
+         return 0;
+      }
+
       @Override
       public void persist(ActiveMQBuffer targetRecord) {
       }
@@ -381,7 +384,7 @@ public class AcknowledgeTest extends ActiveMQTestBase {
       }
 
       @Override
-      public void reloadPersistence(ActiveMQBuffer record) {
+      public void reloadPersistence(ActiveMQBuffer record, CoreMessageObjectPools pools) {
 
       }
 
@@ -392,6 +395,11 @@ public class AcknowledgeTest extends ActiveMQTestBase {
 
       @Override
       public ICoreMessage toCore() {
+         return toCore(null);
+      }
+
+      @Override
+      public ICoreMessage toCore(CoreMessageObjectPools coreMessageObjectPools) {
          return null;
       }
 
@@ -435,22 +443,37 @@ public class AcknowledgeTest extends ActiveMQTestBase {
       }
 
       @Override
-      public int incrementRefCount() throws Exception {
+      public int getUsage() {
          return 0;
       }
 
       @Override
-      public int decrementRefCount() throws Exception {
+      public int usageUp() {
          return 0;
       }
 
       @Override
-      public int incrementDurableRefCount() {
+      public int usageDown() {
          return 0;
       }
 
       @Override
-      public int decrementDurableRefCount() {
+      public int refUp() {
+         return 0;
+      }
+
+      @Override
+      public int refDown() {
+         return 0;
+      }
+
+      @Override
+      public int durableUp() {
+         return 0;
+      }
+
+      @Override
+      public int durableDown() {
          return 0;
       }
 
@@ -483,16 +506,6 @@ public class AcknowledgeTest extends ActiveMQTestBase {
          return null;
       }
 
-      @Override
-      public Message setBuffer(ByteBuf buffer) {
-         return null;
-      }
-
-      @Override
-      public ByteBuf getBuffer() {
-         return null;
-      }
-
       @Override
       public Message setAddress(String address) {
          return null;
@@ -653,6 +666,11 @@ public class AcknowledgeTest extends ActiveMQTestBase {
          return null;
       }
 
+      @Override
+      public Message putStringProperty(SimpleString key, String value) {
+         return null;
+      }
+
       @Override
       public Message putStringProperty(String key, String value) {
          return null;
@@ -812,5 +830,19 @@ public class AcknowledgeTest extends ActiveMQTestBase {
       public Map<String, Object> toPropertyMap() {
          return null;
       }
+
+      @Override
+      public long getPersistentSize() throws ActiveMQException {
+         return 0;
+      }
+
+      @Override
+      public Object getOwner() {
+         return null;
+      }
+
+      @Override
+      public void setOwner(Object object) {
+      }
    }
 }
